/*
 * Copyright (c) 2011-2012, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "stdfarmos.h"
#include "net/seal_comm.h"

#define COMMAND_TO_SEND 17
#define SEQNUM_TO_SEND 3

// send a single command
void appMainSimpleCommand(void)
{
    for (;;) {
        sealCommSendCommand(COMMAND_TO_SEND);
        mdelay(5000);
    }
}

// send command and sequence nunber
void appMainPacket(void)
{
    struct MyPacket_s {
        SealHeader_t header;
        uint32_t fields[2];
    } PACKED;
    typedef struct MyPacket_s MyPacket_t;

    MyPacket_t myPacket;

    for (;;) {
        sealCommPacketStart((SealPacket_t *)&myPacket);
        sealCommPacketAddField(PACKET_FIELD_ID_COMMAND, COMMAND_TO_SEND);
        sealCommPacketAddField(PACKET_FIELD_ID_SEQNUM, SEQNUM_TO_SEND);
        sealCommPacketFinish();

        mdelay(5000);
    }
}

void valueRxCallback(uint16_t code, int32_t value)
{
    PRINTF("received sensor %d data %ld\n", code, value);
}

void appMain(void)
{
    // register interest to all kind of SEAL packets
    uint16_t i;
    for (i = 0; i < 31; ++i) {
        sealCommRegisterInterest(i, valueRxCallback);
    }

//  appMainSimpleCommand();
    appMainPacket();
}
