/**
 * Copyright (c) 2011-2012, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

//----------------------------------------------------
//      Read temperature and humidity sensors, print values to UART
//----------------------------------------------------

#include "stdfarmos.h"
#include <humidity.h>
#include <chips/ds18b20/ds18b20.h>
#include <chips/sht11/sht11_conv.h>
#include <adc.h>
#include <platform.h>

void appMain(void)
{
    uint16_t airTemp, airHum, soilHum;
    int16_t  soilTemp;

    PRINTF("Sensor read test application\n");

    mdelay(100); // Wait for sensors to initialize

    humidityOn();

    while (1) {
        airTemp  = temperatureRead();
        airHum   = humidityRead();
        soilTemp = ds18b20Measure();
        soilHum = adcRead(ADC_SOIL_HUMIDITY);
#if 1
        PRINTF("Air temperature raw: 0x%04x C\n", airTemp);
        PRINTF("Air humidity raw: 0x%04x\n", airHum);
        PRINTF("Soil temperature raw: 0x%04x\n", soilTemp);
        PRINTF("Soil humidity raw: 0x%04x\n", soilHum);
#endif

        PRINTF("Air temperature: %d C\n", sht11_raw2deg_temp(airTemp));
        PRINTF("Air humidity: %d%%\n", sht11_raw2rel_hum(airHum));

        PRINTF("Soil temperature: %d.%04d C\n", soilTemp >> 4, 625 * (soilTemp & 0xF));
        //
        // From the docs: 0.2V is 5%, 2.5V is 50%
        // Reference is 2.5V, ADC output is 0--1023
        // Assume that sensor output is linear, then:
        //
        //  /  5 = a + 1023*0.2/2.5 * b
        // <
        //  \ 50 = a + 1023*2.5/2.5 * b
        //
        // a ~= 1.0870 ~= 4452 / 4096
        // b ~= 0.0478 ~=  196 / 4096
        //
        // (4096 was taken semi-arbitrarily)
        //
        PRINTF("Soil humidity: %ld%%\n\n", (4452L + soilHum * 196L + 2048) / 4096);

        mdelay(3000);
    }
}
