/*
 * Copyright (c) 2011-2012, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

//
// MSP430 User Button
//

#ifndef _USER_BUTTON_HAL_H_
#define _USER_BUTTON_HAL_H_

#include <digital.h>
#include <platform.h>

//===========================================================
// Data types and constants
//===========================================================

enum ButtonState_e {
    BUTTON_RELEASED = 0,
    BUTTON_PRESSED = 1
} PACKED;

typedef enum ButtonState_e ButtonState_t;

typedef void (*ButtonFunc_p)(void);

//===========================================================
// Functions
//===========================================================

static inline void userButtonInit(void) {
    pinAsInput(USER_BUTTON_PORT, USER_BUTTON_PIN);
}

// starts catching user button ints and sets callback
void userButtonEnable(ButtonFunc_p handler);

// disables listening
void userButtonDisable(void);

// returns actual state of user button
ButtonState_t userButtonGet(void);

#endif
