/**
 * Copyright (c) 2011-2012, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * fs/block/dir.h -- directory operations
 */

#ifndef _FS_BLOCK_DIR_H_
#define _FS_BLOCK_DIR_H_

#include <fs/common.h>
#include <fs/types.h>

#include "common.h"

/* Interface functions */
bool  fsBlockStatLast(void * restrict dir, struct fsStat * restrict buf);
bool  fsBlockRemoveLast(void *dir);
bool  fsBlockRenameLast(void * restrict dir, const char * restrict name);
bool  fsBlockCreateAt(void * restrict parent, const char * restrict name,
                      bool dir);

void *fsBlockOpenLastDir(void *parent);
bool  fsBlockReadDir(void * restrict dir, char * restrict name);
void  fsBlockRewindDir(void *dir);
void  fsBlockCloseDir(void *dir);

/* Internal subsystem functions */
struct fsFileControlBlock *fsBlockOpenFcb(struct fsFileControlBlock *dir,
                                          bool write);
void fsBlockCloseFcb(struct fsFileControlBlock *file);
void fsBlockSyncEntry(struct fsFileControlBlock *file);
blk_t fsRootDir(void);

/* init.c needs this */
inline void freeFcb(struct fsFileControlBlock *fcb);

extern struct fsFileControlBlock fcbs[FS_MAX_OPEN_FILES+1];
extern mos_mutex_t fsFcbMutex;

#endif /* _FS_BLOCK_DIR_H */
