/*
 * Copyright (c) 2011-2012, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "random.h"
#include <timing.h>
#include <serial_number.h>

#if PLATFORM_PC
#include <sys/time.h>
#include <stdlib.h>
#endif

static uint32_t randomKey;

void randomInit() {
#if PLATFORM_PC
    struct timeval tv;
    gettimeofday(&tv, NULL);
    randomKey = tv.tv_usec + tv.tv_sec;
#elif PLATFORM_TELOSB || PLATFORM_SADMOTE || PLATFORM_SM3
    // generate a random key from serial number
    uint32_t snum[2];
    serialNumberRead((uint8_t *) snum);
    randomKey = snum[0] + snum[1];
#else
#warning Hardware seed for RNG not implemented!
    randomKey = 0;
#endif
}

void randomSeed(uint16_t seed) {
    randomKey = seed;
}

uint16_t randomNumber(void) {
    //
    // The numbers are from "Numerical Recipes"
    //
    randomKey = randomKey * 1664525ul + 1013904223ul;
    return (uint16_t) (randomKey >> 16);
}
