/*
 * Copyright (c) 2011-2012, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef FARMOS_ASSERT_H
#define FARMOS_ASSERT_H

/// \file
/// The declaration of the ASSERT macro
///

#include <defines.h>

//! Print a message describing the failure, then panic()
void assertionFailed(const char * restrict msg, const char * restrict file,
                     int line) NORETURN;

/// Panic right away
///
/// The function disables interrupts, enters MCU active mode, and blinks all LEDs constantly.
void panic(void) NORETURN;

//! Assert the 'expression' to be true. panic() on failure
extern inline void ASSERT(bool expression);
//! Advanced version of ASSERT macro that works even when the stack pointer is corrupted
extern inline void ASSERT_NOSTACK(bool expression);

//! Warn if the 'expression' IS true
extern inline void WARN_ON(bool expression);

// include the implementation of ASSERT macros
#include <lib/assert_internal.h>

#endif
