/*
 * Copyright (c) 2013, Institute of Electronics and Computer Science
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of  conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

//----------------------------------------------------------
//      Platform code
//----------------------------------------------------------
#include "platform.h"
#include <digital.h>
#include <serial_number.h>
#include "cc1101_pins.h"

#include "sht_pins.h"
#include <chips/ds18b20/ds18b20.h>

#define XREN(port) P##port##REN
#define REN(port)  XREN(port)

//----------------------------------------------------------
//      Init the platform as if on cold reset
//----------------------------------------------------------
void initPlatform(void)
{
    msp430Init();

    pinAsOutput(PLATFORM_SENSORS_VOLTAGE_PORT,          PLATFORM_SENSORS_VOLTAGE_PIN         );
    pinAsOutput(PLATFORM_PERIPHERALS_VOLTAGE_PORT,      PLATFORM_PERIPHERALS_VOLTAGE_PIN     );
    pinAsOutput(PLATFORM_SDI12_ENABLE_PORT,             PLATFORM_SDI12_ENABLE_PIN            );
    pinAsOutput(PLATFORM_UART_SDI12_SWITCH_PORT,        PLATFORM_UART_SDI12_SWITCH_PIN       );
    pinAsOutput(PLATFORM_RADIATION_CURRENT_SWITCH_PORT, PLATFORM_RADIATION_CURRENT_SWITCH_PIN);
    pinAsOutput(PLATFORM_TEMP_HUMIDITY_SWITCH_PORT,     PLATFORM_TEMP_HUMIDITY_SWITCH_PIN    );

    platformSelectCurrent();
    platformSelectSDI12();
    platformEnablePeripherals(); // For radio
    platformDisableSensors(); // Disables UART too

#if USE_ADC
    platformEnableADC(ADC_RADIATION_CURRENT);
    platformEnableADC(ADC_AIR_TEMP_HUMIDITY);
    platformEnableADC(ADC_LIGHT_CHANNEL);
    platformEnableADC(ADC_AIR_PRESSURE);
    platformEnableADC(ADC_SOIL_MOISTURE);
    platformEnableADC(ADC_BATTERY_VOLTAGE);
    platformEnableADC(ADC_RESERVED);
    platformEnableADC(ADC_WIND_DIR);

    hplAdcUse1_5VRef(); // miimote uses 1.5V reference
#endif

#if USE_SERIAL_NUMBER
    int retries = 10;
    do
    {
        // Set up Vcc for the DS2411
        pinAsOutput(DS2411_VCC_PORT, DS2411_VCC_PIN);
        pinSet(DS2411_VCC_PORT, DS2411_VCC_PIN);

        // Enable the pullup resistor
        REN(DS2411_PORT) |= BV(DS2411_PIN);

        // Read the serial number
        serialNumberInit();

        // Disable Vcc
        pinClear(DS2411_VCC_PORT, DS2411_VCC_PIN);

        if (ds2411Ok)
        {
            break;
        }
        else
        {
            mdelay(100);
        }
    }
    while (--retries > 0);
#endif

    // enable pullup resistor for soil temperature sensor
    // Disabled in favor of external pullup resistor
    // REN(DS18B20_PORT) |= BV(DS18B20_PIN);

    ds18b20Init(); // Return value ignored

    // enable pullup resistor for SHT sensor
    // Disabled in favor of external pullup resistor
    //REN(SHT11_SDA_PORT) |= BV(SHT11_SDA_PIN);
}
